<template>
  <ContentWrap>
    <!-- 搜索工作栏 -->
    <el-form
      class="-mb-15px"
      :model="queryParams"
      ref="queryFormRef"
      :inline="true"
      label-width="68px"
    >
      <el-form-item label="主机名称" prop="hostName">
        <el-select v-model="queryParams.hostName" clearable placeholder="请选择" class="!w-180px" @change="getDataList">
          <el-option
            v-for="(host, index) in hosts"
            :key="index"
            :label="host"
            :value="host"
          />
        </el-select>
      </el-form-item>
      <el-form-item label="服务器IP" prop="hostIp">
        <el-select v-model="queryParams.hostIp" clearable placeholder="请选择" class="!w-160px" @change="getDataList">
          <el-option
            v-for="(ip, index) in ips"
            :key="index"
            :label="ip"
            :value="ip"
          />
        </el-select>
      </el-form-item>
      <el-form-item label="创建时间" prop="createTime">
        <el-date-picker
          v-model="queryParams.createTime"
          value-format="YYYY-MM-DD HH:mm:ss"
          type="datetimerange"
          start-placeholder="开始日期"
          end-placeholder="结束日期"
          :default-time="[new Date('1 00:00:00'), new Date('1 23:59:59')]"
          class="!w-360px"
        />
      </el-form-item>
      <el-form-item>
        <el-button @click="handleQuery">
          <Icon icon="ep:search" class="mr-5px"/>
          搜索
        </el-button>
        <el-button @click="resetQuery">
          <Icon icon="ep:refresh" class="mr-5px"/>
          重置
        </el-button>
        <el-button
          type="primary"
          plain
          @click="openForm('create')"
          v-hasPermi="['infra:monitor-disk:create']"
        >
          <Icon icon="ep:plus" class="mr-5px"/>
          新增
        </el-button>
        <el-button
          type="success"
          plain
          @click="handleExport"
          :loading="exportLoading"
          v-hasPermi="['infra:monitor-disk:export']"
        >
          <Icon icon="ep:download" class="mr-5px"/>
          导出
        </el-button>
      </el-form-item>
      <el-form-item style="float: right">
        <el-button
          v-if="showType == 'chart'"
          type="warning"
          style="font-weight: bold"
          plain
          @click="switchShow('data')">
          <Icon icon="fa-solid:th-list" class="mr-5px"/>
          列表展示
        </el-button>
        <el-button
          v-if="showType == 'data'"
          type="danger"
          style="font-weight: bold"
          plain
          @click="switchShow('chart')">
          <Icon icon="fa-solid:chart-pie" class="mr-5px"/>
          图例展示
        </el-button>
      </el-form-item>
    </el-form>
  </ContentWrap>

  <ContentWrap v-if="showType == 'chart'">
    <!-- 磁盘使用率折线图 -->
    <div id="chartArea"></div>

    <!-- 磁盘使用率饼图 -->
    <h3 style="margin-top: 20px; margin-bottom: 10px">主机磁盘使用率</h3>
    <div v-for="host in hostList" :key="host.name" class="host">
      <div class="host-child">
        <h4>主机名：{{ host.name }}&nbsp;&nbsp;&nbsp;&nbsp;主机IP：{{ host.ip }}</h4>
        <el-skeleton :loading="echartsLoading" animated>
          <div class="disks">
            <div v-for="disk in host.disks" :key="disk.name" class="disk">
              <h4 id="diskTitle">{{ disk.disk }}</h4>
              <PieChart :used="disk.used" :total="disk.total" />
              <div class="disk-info">
                <div style="margin-bottom: 6px; font-size: 16px"><span style="color: #b9292b ;font-weight: bolder">{{ disk.total != 0 ? ((disk.used / disk.total) * 100).toFixed(1) : 0.0 }}% </span>已使用</div>
                <div style="font-weight: bolder">{{ disk.used }}GB / {{ disk.total }}GB</div>
              </div>
            </div>
          </div>
        </el-skeleton>
      </div>
    </div>
  </ContentWrap>

  <!-- 列表 -->
  <ContentWrap v-if="showType == 'data'">
    <el-table v-loading="loading" :data="list" :stripe="true" :show-overflow-tooltip="true">
      <el-table-column label="主机名称" align="center" prop="hostName"/>
      <el-table-column label="服务器ip" align="center" prop="hostIp"/>
      <el-table-column label="盘符" align="center" prop="disk"/>
      <el-table-column label="磁盘名" align="center" prop="diskName"/>
      <el-table-column label="总空间" align="center" prop="spaceTotal"/>
      <el-table-column label="已用空间" align="center" prop="spaceUsed"/>
      <el-table-column label="可用空间" align="center" prop="spaceUsable"/>
      <el-table-column label="空间使用比例" align="center" prop="spaceRatio"/>
      <el-table-column
        label="创建时间"
        align="center"
        prop="createTime"
        :formatter="dateFormatter"
        width="180px"
      />
      <el-table-column label="操作" align="center">
        <template #default="scope">
          <el-button
            link
            type="primary"
            @click="openForm('update', scope.row.id)"
            v-hasPermi="['infra:monitor-disk:query']"
          >
            详情
          </el-button>
          <el-button
            link
            type="danger"
            @click="handleDelete(scope.row.id)"
            v-hasPermi="['infra:monitor-disk:delete']"
          >
            删除
          </el-button>
        </template>
      </el-table-column>
    </el-table>
    <!-- 分页 -->
    <Pagination
      :total="total"
      v-model:page="queryParams.pageNo"
      v-model:limit="queryParams.pageSize"
      @pagination="getList"
    />
  </ContentWrap>

  <!-- 表单弹窗：添加/修改 -->
  <MonitorDiskForm ref="formRef" @success="getList"/>
</template>

<script setup lang="ts">
import {dateFormatter} from '@/utils/formatTime'
import download from '@/utils/download'
import {MonitorDiskApi, MonitorDiskVO} from '@/api/infra/monitordisk'
import MonitorDiskForm from './MonitorDiskForm.vue'
import {EChartsOption} from "echarts";
import * as echarts from 'echarts';
import {formatTime} from "@/utils";
import {formatDate} from "@vueuse/core";
import PieChart from '@/components/MonitorDiskPie/PieChart.vue';

/** 磁盘监控日志 列表 */
defineOptions({name: 'MonitorDisk'})

const message = useMessage() // 消息弹窗
const {t} = useI18n() // 国际化

const loading = ref(true) // 列表的加载中
const list = ref<MonitorDiskVO[]>([]) // 列表的数据
const hosts = ref<String[]>([]) // 主机列表
const ips = ref<String[]>([]) // ip列表
const total = ref(0) // 列表的总页数
const queryParams = reactive({
  pageNo: 1,
  pageSize: 10,
  hostName: undefined,
  hostIp: undefined,
  disk: undefined,
  diskName: undefined,
  spaceTotal: undefined,
  spaceUsed: undefined,
  spaceUsable: undefined,
  spaceRatio: undefined,
  createTime: [],
})
const queryFormRef = ref() // 搜索的表单
const exportLoading = ref(false) // 导出的加载中
const echartsLoading = ref(true) // 图表加载中
const showType = ref() //展示类型（chart-图例，data-数据）

const hostList = ref([]);


// 颜色集合（10个区分度较好的颜色）
const colors = [
  '#5470C6', '#91CC75', '#FAC858', '#EE6666', '#73C0DE',
  '#3BA272', '#FC8452', '#9A60B4', '#EA7CCC', '#19DCDC'
]
const chartRefs = ref<HTMLElement[]>([])
// 图表实例和容器管理
const chartInstances = ref<{ instance: echarts.ECharts; container: HTMLDivElement }[]>([])

// 清理所有图表资源
const cleanCharts = () => {
  // 1. 销毁所有图表实例
  chartInstances.value.forEach(({ instance }) => {
    instance.dispose()
  })

  // 2. 移除所有容器元素
  chartInstances.value.forEach(({ container }) => {
    container.remove()
  })

  // 3. 清空实例记录
  chartInstances.value = []
}


// 处理接口数据
const processServerData = (apiData: any[]) => {
  return apiData.flatMap(serverObj => {
    return Object.entries(serverObj).map(([serverName, dataPoints]) => ({
      serverName,
      data: (dataPoints as any[]).map(item => ({
        createTime: item.createTime,
        ...Object.fromEntries(
          Object.entries(item)
            .filter(([key]) => key !== 'createTime')
            .map(([key, value]) => [key.replace(/[()/]/g, '_'), value]) // 清理特殊字符
        )
      })).sort((a, b) => a.createTime - b.createTime) // 按时间排序
    }))
  })
}

// 初始化图表
const initCharts = async (apiData: any[]) => {
  cleanCharts()
  await nextTick()

  const servers = processServerData(apiData)
  const chartArea = document.querySelector('#chartArea')

  servers.forEach((server, index) => {
    // 创建容器
    const container = document.createElement('div')
    container.style.width = '100%'
    container.style.height = '300px'
    container.classList.add('chart-container')
    chartArea?.appendChild(container)

    // 初始化图表实例
    const chart = echarts.init(container)
    const dimensions = Object.keys(server.data[0]).filter(k => k !== 'createTime')

    // 图表配置
    const option: echarts.EChartsOption = {
      title: {
        text: `${server.serverName} 磁盘使用率趋势`,
        left: 'center',
        textStyle: {
          fontSize: 16,
          fontWeight: 'bold'
        }
      },
      tooltip: {
        trigger: 'axis',
        valueFormatter: (value) => `${value}%`,
        axisPointer: {
          type: 'cross',
          label: {
            backgroundColor: '#6a7985'
          }
        }
      },
      legend: {
        type: 'scroll',
        top: 30,
        pageIconColor: '#2c3e50',
        pageTextStyle: {
          color: '#666'
        }
      },
      grid: {
        top: 80,
        left: 50,
        right: 30,
        bottom: 30,
        containLabel: true
      },
      xAxis: {
        type: 'time',
        axisLabel: {
          formatter: (value: number) => {
            const date = new Date(value)
            return `${date.getMonth() + 1}/${date.getDate()}`
          },
          interval: 0,
          length: 6,
          rotate: 45,
          fontSize: 12
        },
        min: (value) => value.min - 86400000,
        max: (value) => value.max + 86400000
      },
      yAxis: {
        type: 'value',
        min: 0,
        max: 100,
        axisLabel: {
          formatter: '{value}%',
          fontSize: 12
        },
        splitLine: {
          show: true,
          lineStyle: {
            type: 'dashed'
          }
        }
      },
      toolbox: {
        feature: {
          dataZoom: {
            type: 'inside',
            filterMode: 'none', // 禁用数据过滤
            yAxisIndex: false,
            title: { zoom: '缩放', back: '还原' }
          },
          brush: {
            type: ['lineX', 'clear'],
            title: { lineX: '选择', clear: '清除' }
          },
          saveAsImage: {
            name: '磁盘使用率图表',
            title: '保存',
            pixelRatio: 2
          }
        }
      },
      color: colors,
      series: dimensions.map((dim, dimIndex) => ({
        name: dim.replace(/_/g, ' '), // 还原清理的特殊字符
        type: 'line',
        // 关键配置项
        progressive: 0,          // 禁用分片渲染
        large: false,            // 禁用大数据模式
        showAllSymbol: true,     // 显示所有数据点
        sampling: 'none',        // 禁用采样
        // 数据维度声明
        dimensions: ['createTime', 'value'],
        smooth: true,
        symbol: 'none',
        areaStyle: {
          opacity: 0.3,
          color: new echarts.graphic.LinearGradient(0, 0, 0, 1, [
            { offset: 0, color: colors[dimIndex % colors.length] },
            { offset: 1, color: 'rgba(255,255,255,0)' }
          ])
        },
        lineStyle: {
          width: 2,
          color: colors[dimIndex % colors.length]
        },
        data: server.data.map(d => [d.createTime, d[dim]])
      })),
      dataZoom: [{
        type: 'inside',
        filterMode: 'none' // 禁用数据过滤
        // type: 'inside',
        // start: 0,
        // end: 100,
        // minValueSpan: 86400000 * 1 // 最小缩放范围为1天
      }]
    }

    chart.setOption(option)
    chartInstances.value.push({ instance: chart, container })
  })
}


/** 查询列表 */
const getList = async () => {
  loading.value = true
  try {
    const data = await MonitorDiskApi.getMonitorDiskPage(queryParams)
    list.value = data.list
    total.value = data.total
  } finally {
    loading.value = false
  }
}

/** 查询所有主机名 */
const getAllHost = async () => {
  const data = await MonitorDiskApi.getAllHost()
  hosts.value = data
}

/** 查询所有ip */
const getAllIp = async () => {
  const data = await MonitorDiskApi.getAllIp()
  ips.value = data
}


/** 搜索按钮操作 */
const handleQuery = () => {
  queryParams.pageNo = 1
  if (showType.value == 'data') {
    getList()
  } else {
    getMonitorDiskDataList()
    usedDiskInstance()
  }
}

/** 重置按钮操作 */
const resetQuery = () => {
  queryFormRef.value.resetFields()
  handleQuery()
}

/** 添加/修改操作 */
const formRef = ref()
const openForm = (type: string, id?: number) => {
  formRef.value.open(type, id)
}

/** 删除按钮操作 */
const handleDelete = async (id: number) => {
  try {
    // 删除的二次确认
    await message.delConfirm()
    // 发起删除
    await MonitorDiskApi.deleteMonitorDisk(id)
    message.success(t('common.delSuccess'))
    // 刷新列表
    await getList()
  } catch {
  }
}

/** 导出按钮操作 */
const handleExport = async () => {
  try {
    // 导出的二次确认
    await message.exportConfirm()
    // 发起导出
    exportLoading.value = true
    const data = await MonitorDiskApi.exportMonitorDisk(queryParams)
    download.excel(data, '磁盘监控日志.xls')
  } catch {
  } finally {
    exportLoading.value = false
  }
}

/** 查询统计数据列表 */
const getMonitorDiskDataList = async () => {
  const list = await MonitorDiskApi.getMonitorDiskList(queryParams)
  echartsLoading.value = false
  await initCharts(list)
}

const usedDiskInstance = async () => {
  const list = await MonitorDiskApi.getMonitorDiskInfo(queryParams)
  hostList.value = list
}

/** 封装接口调用 */
const getDataList = async () => {
  if(showType.value == 'data') {
    await getList()
  } else {
    await getMonitorDiskDataList()
    await usedDiskInstance()
  }
}

/** 切换展示方式 */
const switchShow = (type: String) => {
  showType.value = type
  if (showType.value == 'data') {
    getList()
  } else {
    getMonitorDiskDataList()
    usedDiskInstance()
  }
}

let intervalId;
/** 初始化 **/
onMounted(() => {
  showType.value = 'data';
  getAllHost();
  getAllIp();
  getDataList()
  intervalId = setInterval(() => {
    getDataList()
  }, 300000);
})

onBeforeUnmount(() => {
  cleanCharts()
})

onUnmounted(() => {
  clearInterval(intervalId);
});
</script>
<style>
  .host {
    margin-bottom: 20px;
    margin-right: 20px;
    border-radius: 8px;
  }
  .host-child {
    background: rgba(200, 200, 200, 0.3);
    border-radius: 8px;
    padding: 10px;
  }
  .disks {
    display: grid;
    grid-template-columns: repeat(auto-fill, minmax(210px, 1fr));
    gap: 20px;
    margin-top: 20px;
  }
  .disk {
    width: 250px;
    background: rgba(100, 100, 150, 0.2);
    padding: 15px;
    border-radius: 16px;
    text-align: center;
    box-shadow: 0 2px 4px rgba(0,0,0,0.1);
  }

  #diskTitle {
    margin-bottom: 10px
  }

  .disk-info {
    margin-top: 10px;
    font-size: 0.9em;
    color: #666;
  }

</style>
